#include <avr/io.h>
#include <Arduino.h>

// Adjustable parameters
unsigned long desiredFrequency = 2000000; // PWM frequency in Hz
float dutyCyclePercentage = 33.0;         // Duty cycle in percentage
unsigned int burstCycles = 100;           // Number of PWM cycles per burst
float offPeriodMs = 0.5;                  // "Off" period in milliseconds between bursts

void setup() {
    Serial.begin(9600);
    Serial.println("Starting pulse train PWM on pin 9 with adjustable parameters.");
    Serial.println("Detailed Signal Information:");

    // Configure pin 9 (OC1A, PB1) as output
    DDRB |= (1 << DDB1);

    // Set Timer1 for Fast PWM, non-inverting on OC1A, no prescaler
    TCCR1A = (1 << WGM11) | (1 << COM1A1);  // Fast PWM, non-inverting on OC1A
    TCCR1B = (1 << WGM13) | (1 << WGM12) | (1 << CS10);  // Mode 14, no prescaler

    // Set the PWM frequency and duty cycle
    setPWMPin9(desiredFrequency, dutyCyclePercentage);

    // Print initial signal details
    printSignalDetails();
}

void loop() {
    // Enable PWM for the burst duration
    for (unsigned int i = 0; i < burstCycles; i++) {
        TCCR1A |= (1 << COM1A1);  // Re-enable PWM output on OC1A (pin 9)
        delayMicroseconds(1000000 / desiredFrequency);  // Wait for one PWM cycle
    }

    // Disable PWM by disconnecting OC1A output
    TCCR1A &= ~(1 << COM1A1);  // Disable PWM output temporarily
    PORTB &= ~(1 << PORTB1);   // Ensure pin is set to LOW during "off" period

    // Delay for the "off" period, supporting sub-millisecond values
    delayMicroseconds(offPeriodMs * 1000);  // Convert milliseconds to microseconds
}

// Function to calculate and apply frequency and duty cycle
void setPWMPin9(unsigned long frequency, float dutyCycle) {
    // Calculate ICR1 (TOP) based on desired frequency
    unsigned int topValue = (16000000 / frequency) - 1;  // Assuming 16 MHz clock
    ICR1 = topValue;

    // Set OCR1A based on duty cycle percentage
    unsigned int compareValue = (dutyCycle / 100.0) * ICR1;
    OCR1A = compareValue;
}

// Function to print detailed signal information to the Serial Monitor
void printSignalDetails() {
    Serial.println("\n--- PWM Signal Details ---");

    // Frequency and duty cycle
    Serial.print("Frequency: ");
    Serial.print(desiredFrequency);
    Serial.println(" Hz");

    Serial.print("Duty Cycle: ");
    Serial.print(dutyCyclePercentage);
    Serial.println(" %");

    // Timer TOP and OCR1A values
    Serial.print("ICR1 (TOP): ");
    Serial.println(ICR1);
    
    Serial.print("OCR1A (Duty Cycle): ");
    Serial.println(OCR1A);

    // Calculating on-time for each pulse
    float cycleTimeMicroseconds = (1.0 / desiredFrequency) * 1000000;
    float onTimePerCycle = cycleTimeMicroseconds * (dutyCyclePercentage / 100.0);

    Serial.print("On-Time per PWM cycle: ");
    Serial.print(onTimePerCycle);
    Serial.println(" µs");

    Serial.print("Total On-Time for burst: ");
    Serial.print(onTimePerCycle * burstCycles);
    Serial.println(" µs");

    // Calculating total off-time
    Serial.print("Off-Time between bursts: ");
    Serial.print(offPeriodMs);
    Serial.println(" ms");

    // Total period for each pulse train cycle
    float totalCyclePeriod = (onTimePerCycle * burstCycles / 1000.0) + offPeriodMs;

    Serial.print("Total Period of Pulse Train Cycle: ");
    Serial.print(totalCyclePeriod);
    Serial.println(" ms");

    Serial.println("--- End of Signal Details ---\n");
}
